/*
 * ClockEveryWhere Hack
 *  
 * Put into GPL licence by Renaud malaval <renaud.malaval2@fnac.net>
 *  
 * File
 * ----
 * utils.c
 *  
 * History
 * -------
 * 15/06/00 Created
*/


#include <PalmOS.h>
#include <StringMgr.h>
#include <Form.h>
#include <TextMgr.h>
#include <Window.h>
#include <TimeMgr.h>
#include <FeatureMgr.h>
#include <Menu.h>

// include system use only for test
#include <SoundMgr.h>

#include <SysEvtMgr.h>
#include <HandEra/Silk.h>

#include <Table.h>
#include <HandEra/Vga.h>

#include "ResourceDefines.h"
#include "ClockEveryWhere.h"
#include "utils.h"

#define TRACE_OUTPUT		TRACE_OUTPUT_OFF

static const UInt16		HourHandX[12] = {5, 7, 8, 8, 8, 7, 5, 3, 2, 2, 2, 3 };
static const UInt16		HourHandY[12] = {2, 2, 3, 5, 7, 8, 8, 8, 7, 5, 3, 2 };

static const UInt16		MinuteHandX[12] = {5, 8, 10, 10, 10, 8, 5, 2, 0, 0, 0, 2 };
static const UInt16		MinuteHandY[12] = {0, 0, 2, 5, 8, 10, 10, 10, 8, 5, 2, 0 };

static const RGBColorType	WhiteColor = {255, 255, 255, 255};

void * GetObjectPtr( FormPtr theFormP, UInt16 objectID)
{
	if (!theFormP )
		theFormP = FrmGetActiveForm();
	return (FrmGetObjectPtr(theFormP, FrmGetObjectIndex(theFormP, objectID)));
}	/* GetObjectPtr */


void Utils_GetHackPref(CEWpref_t *aHackPrefsPtr)
{
	UInt32		theDepth;
	UInt16		thePrefSize = sizeof(CEWpref_t);
	Int16			theErr;

	theErr = PrefGetAppPreferences(kCreatorCode, kPrefIdNum, aHackPrefsPtr, &thePrefSize, true);
	// Remember the error could be kVersionNum (the pref version num)
	if ((theErr == noPreferenceFound) || (theErr != kVersionNum))
	{
		aHackPrefsPtr->DisplayIfNameIsNumberControlvalue = false;
		aHackPrefsPtr->DisplayAlertClockControlvalue = true;
		aHackPrefsPtr->BlinkingSecondControlvalue = false;
		aHackPrefsPtr->AnalogClockControlvalue = false;
		aHackPrefsPtr->WhiteBackgroundControlvalue = true;
		aHackPrefsPtr->DisplayAlertClockRightControlvalue = false;
		aHackPrefsPtr->DisplayAnalogOn12Hour = 0;
		aHackPrefsPtr->Add0Before1To9 = false;
		aHackPrefsPtr->FontInBold = 0;

		WinScreenMode (winScreenModeGet, NULL, NULL, &theDepth, NULL);
		if (theDepth > 4)
			aHackPrefsPtr->DisplayIsInColor = (UInt8)theDepth;
		else
			aHackPrefsPtr->DisplayIsInColor = 0;

		PrefSetAppPreferences (kCreatorCode, kPrefIdNum, kVersionNum, aHackPrefsPtr, sizeof(*aHackPrefsPtr), true);
	}
}	/* Utils_GetAppPref */


void Utils_DrawTheClock(FormPtr frmP, Boolean force, CEWpref_t *aHackPrefsPtr)
{
	DateTimeType				theToday;
	Char							*theFrmTitleP;
	MenuBarType					*theMenuBarP;
	UInt32						theCurrentValue;
	UInt32						thePreviewValue = 0;
   UInt32						theVersion;
	FontID						theStdFont;
	UInt16						thePosX;
	UInt16						thePosY;
	UIColorTableEntries		theWindowType_frmAlrMdl;
	Boolean						theFormIsAnAlert = false;
   Boolean						theHanderaHighRes = false;

	if (_TRGVGAFeaturePresent(&theVersion))
	{
      VgaScreenModeType mode;
      VgaRotateModeType rotate;
      VgaGetScreenMode(&mode, &rotate);
		if (mode == screenMode1To1)
			theHanderaHighRes = true;
      else
			theHanderaHighRes = false;
    }
    else
		 theHanderaHighRes = false;

	theFrmTitleP = (char *)FrmGetTitle (frmP);
	if ((theFrmTitleP) && (*theFrmTitleP != '\0'))
	{
		theMenuBarP = MenuGetActiveMenu();
		if (theMenuBarP && theMenuBarP->attr.visible)
			return;												/* if Menu is displaying then exit */

		if (WinModal ( FrmGetWindowHandle (frmP)))
		{
			if (!aHackPrefsPtr->DisplayAlertClockControlvalue)
				return;
			thePosX = kInAlertClockPositionX;
			thePosY = kInAlertClockPositionY;	/* don't change this value without see below its use */
			theFormIsAnAlert = true;
			theWindowType_frmAlrMdl = UIDialogFrame;
//			theWindowType_frmAlrMdl = UIAlertFrame;
		}
		else
		{	// compute form title len in pixel to determine where to begin of drawing the clock
			thePosY = kGlobalClockPositionY;
			if (theHanderaHighRes)
			{
				theStdFont = FntSetFont (VgaBaseToVgaFont(boldFont) );
				thePosX = FntCharsWidth (theFrmTitleP, StrLen( theFrmTitleP)) + 6*3/2;
			}
			else
			{
				theStdFont = FntSetFont (boldFont );
				thePosX = FntCharsWidth (theFrmTitleP, StrLen( theFrmTitleP)) + 6;
			}
			FntSetFont (theStdFont );
			theWindowType_frmAlrMdl = UIFormFrame;

			// Special patch for mail.prc to display clock clearly.
			// FntCharsWidth (" 000 Messages", StrLen( " 000 Messages")) = 59
//			if (*((UInt32 *)theFrmTitleP) == *((UInt32 *)"Mail"))	// custom for Palm Mail.prc application
//				thePosX += (theHanderaHighRes) ? 59 * 3/2: 59;
		}

		if (((*theFrmTitleP < '0') || (*theFrmTitleP > '9')) || (aHackPrefsPtr->DisplayIfNameIsNumberControlvalue))
		{
#ifdef BLACKLIST			
			if (!theFormIsAnAlert)		/* BIG problem app can have multiple form with diffrent name */
			{		// black list of bad app 
				Char *	testP;
				UInt32	appName;
				
				testP = theFrmTitleP;
				appName =  (UInt32)(*(testP  )) << 24;
				appName |= (UInt32)(*(++testP)) << 16;
				appName |= (UInt32)(*(++testP)) << 8;
				appName |= (UInt32)(*(++testP));			
				switch (appName)
				{
				case 'BigC':		// BigClock
				case 'iSil':		// iSilo
				case 'iWay':		// iWay-Info
				case 'mail':		// old palm mail
				case 'Shad':		// ShadowPlan
					return;
					break;
				}
			}		
#endif
			FtrGet (kCreatorCode, kFeatureNumSecond, &thePreviewValue);

			TimSecondsToDateTime (TimGetSeconds(), &theToday);

      	if (aHackPrefsPtr->BlinkingSecondControlvalue)
				theCurrentValue = theToday.second;
			else
				theCurrentValue = theToday.minute;

			if ((thePreviewValue != theCurrentValue) || (force))
			{
				if (aHackPrefsPtr->AnalogClockControlvalue)				/* ANALOG clock */
				{
					RectangleType		theClockRec;
					IndexedColorType	theBackColor = 0;
					IndexedColorType	theForeColor = 0;
					UInt16				theHourIndex = 0;
					UInt16				theMinuteIndex = (UInt16)(theToday.minute) / (UInt16)5;
					UInt16				theClockCenterX;
					UInt16				theClockCenterY;
               UInt16				theClockWidth;
               UInt16            theClockHeight;

               if (theHanderaHighRes)
               {
               	theClockWidth = kClockWidth*3/2;
               	theClockHeight = kClockHeight*3/2;
               }
               else
               {
               	theClockWidth = kClockWidth;
               	theClockHeight = kClockHeight;
               }
					if (theToday.minute > 34)
					{
						theToday.hour++;
						if (theToday.hour > 23)
							theToday.hour = 0;
					}
					if (theToday.hour > 11)
						theHourIndex = theToday.hour - 12;
					else
						theHourIndex = theToday.hour;

					if (thePosY)
					{
						thePosX++;
						thePosY--;
					}
					if ((theFormIsAnAlert) && (aHackPrefsPtr->DisplayAlertClockRightControlvalue))
						thePosX = kScreenWidth - (theClockWidth + thePosX + 3); 

					if ((aHackPrefsPtr->WhiteBackgroundControlvalue) && (theFormIsAnAlert))
					{
						if (aHackPrefsPtr->DisplayIsInColor)
						{
						theBackColor = WinSetBackColor (UIColorGetTableEntryIndex ( theWindowType_frmAlrMdl));
							theForeColor = WinSetForeColor (WinRGBToIndex ( &WhiteColor));					
						}
						else
						{
							theBackColor = WinSetBackColor (1);
							theForeColor = WinSetForeColor (theBackColor);		
							WinSetBackColor(theForeColor );
						}
					}

					RctSetRectangle (&theClockRec, thePosX, thePosY, theClockWidth, theClockHeight);
					WinEraseRectangle (&theClockRec, 0);		

               if (theHanderaHighRes)
               {	// Draw the clock frame
						WinDrawPixel ( MinuteHandX[0]*3/2 + thePosX, MinuteHandY[0]*3/2 + thePosY);
						WinDrawPixel ( MinuteHandX[3]*3/2 + thePosX, MinuteHandY[3]*3/2 + thePosY);
						WinDrawPixel ( MinuteHandX[6]*3/2 + thePosX, MinuteHandY[6]*3/2 + thePosY);
						WinDrawPixel ( MinuteHandX[9]*3/2 + thePosX, MinuteHandY[9]*3/2 + thePosY);
						// Draw the needles
						theClockCenterX = (kClockAxisX*3/2) + thePosX;
						theClockCenterY = (kClockAxisY*3/2) + thePosY;
						//hour
						WinDrawLine (theClockCenterX,theClockCenterY,
										HourHandX[ theHourIndex ]*3/2 + thePosX,
										HourHandY[ theHourIndex ]*3/2 + thePosY);
						//minute
						WinDrawLine (theClockCenterX,theClockCenterY,
										MinuteHandX[ theMinuteIndex ]*3/2 + thePosX,
										MinuteHandY[ theMinuteIndex ]*3/2 + thePosY);
               }
               else
               {	// Draw the clock frame
						WinDrawPixel ( MinuteHandX[0] + thePosX, MinuteHandY[0] + thePosY);
						WinDrawPixel ( MinuteHandX[3] + thePosX, MinuteHandY[3] + thePosY);
						WinDrawPixel ( MinuteHandX[6] + thePosX, MinuteHandY[6] + thePosY);
						WinDrawPixel ( MinuteHandX[9] + thePosX, MinuteHandY[9] + thePosY);
						// Draw the needles
						theClockCenterX = kClockAxisX + thePosX;
						theClockCenterY = kClockAxisY + thePosY;
	               //hour
						WinDrawLine (theClockCenterX,	theClockCenterY,
							HourHandX[ theHourIndex] + thePosX, HourHandY[ theHourIndex] + thePosY);
						//minute
						WinDrawLine (theClockCenterX, theClockCenterY,
							MinuteHandX[ theMinuteIndex ] + thePosX, MinuteHandY[ theMinuteIndex ] + thePosY);
					}

					if ((aHackPrefsPtr->WhiteBackgroundControlvalue) && (theFormIsAnAlert))
					{
						WinSetBackColor (theBackColor);
						WinSetForeColor (theForeColor);
					}
				}	/* end of ANALOG clock drawing */
				else				
				{	/* NUMERIC clock */
					Char 		theTimeString[timeStringLength];
					UInt16	theStringLen;
					FontID 	theFontId;

					// numeric format 24h, 12h, AMPM, AP
					if (aHackPrefsPtr->DisplayAnalogOn12Hour)
						TimeToAscii (theToday.hour, theToday.minute, tfColonAMPM, theTimeString);
					else
						TimeToAscii (theToday.hour, theToday.minute, tfColon24h, theTimeString);

					theStringLen = StrLen( theTimeString);
					switch (aHackPrefsPtr->DisplayAnalogOn12Hour)
					{
					case 0:	// 24h
//						theStringLen = (theStringLen > 5) ? 5: theStringLen;
						break;
					case 1:	// 12h remove at the end the " PM" or the " AM"
						theStringLen -= 3;	
						break;
					case 2:	// 12h remove the last space in time string
						theTimeString[theStringLen-3] = theTimeString[theStringLen-2];
						theTimeString[theStringLen-2] = theTimeString[theStringLen-1];
						theStringLen--;	
						break;
					case 3:	// 12h remove the last space and the last char "M" in time string
						theTimeString[theStringLen-3] = theTimeString[theStringLen-2];
						theStringLen -= 2;	
						break;
					}
					theTimeString[theStringLen] = '\0';

      			if (aHackPrefsPtr->BlinkingSecondControlvalue)
					{
						if (theTimeString[1] == ':')
						{
							if (theCurrentValue & 1) theTimeString[1] = ' ';
						}
						else if (theTimeString[2] == ':')
						{
							if (theCurrentValue & 1) theTimeString[2] = ' ';
						}
					}

					if ( aHackPrefsPtr->Add0Before1To9 &&
						((theTimeString[1] == ':') || (theTimeString[1] == ' ')) )
					{
						UInt16 index;
						for (index = theStringLen; index != 0xFFFF; index--)
							theTimeString[index + 1] = theTimeString[index];
						theTimeString[0] = '0';
						theStringLen++;
					}
						
					if ((theFormIsAnAlert) && (aHackPrefsPtr->DisplayAlertClockRightControlvalue))
						thePosX = kScreenWidth - (FntCharsWidth (theTimeString, theStringLen) + thePosX + 3); 

					if (aHackPrefsPtr->FontInBold)
						theFontId = boldFont;
					else
						theFontId = stdFont;				
					if (theHanderaHighRes)
						theStdFont = FntSetFont (VgaBaseToVgaFont (theFontId));
					else
						theStdFont = FntSetFont (theFontId );	
					if (thePosY)
					{	// draw clock near a form title	
						if (aHackPrefsPtr->DisplayIsInColor)
						{
							IndexedColorType	thePreviousBackColorIndex;
							IndexedColorType	thePreviousTextColorIndex;

							thePreviousBackColorIndex = WinSetBackColor ( WinRGBToIndex ( &WhiteColor)); 
							thePreviousTextColorIndex = WinSetTextColor ( UIColorGetTableEntryIndex ( theWindowType_frmAlrMdl));
							WinDrawChars(theTimeString, theStringLen, thePosX, thePosY);
							WinSetBackColor ( thePreviousBackColorIndex);
							WinSetTextColor ( thePreviousTextColorIndex);
						}
						else
							WinDrawChars(theTimeString, theStringLen, thePosX, thePosY);
					}
					else
					{	// draw clock in an alert or a modal
						if (aHackPrefsPtr->DisplayIsInColor)
						{
							IndexedColorType	thePreviousBackColorIndex;
							IndexedColorType	thePreviousTextColorIndex;

							thePreviousBackColorIndex = WinSetBackColor ( UIColorGetTableEntryIndex ( theWindowType_frmAlrMdl));
							thePreviousTextColorIndex = WinSetTextColor ( WinRGBToIndex ( &WhiteColor)); 
							WinDrawChars(theTimeString, theStringLen, thePosX, thePosY);
							WinSetBackColor ( thePreviousBackColorIndex);
							WinSetTextColor ( thePreviousTextColorIndex);
						}
						else
							WinDrawInvertedChars (theTimeString, theStringLen, thePosX, thePosY);
					}
					FntSetFont(theStdFont);
				}	/* end of NUMERIC clock drawing */
				FtrSet (kCreatorCode, kFeatureNumSecond, theCurrentValue);
			}
		}
	}
}	/* Utils_DrawTheClock */

/* Function for OS compatibility */

Err Utils_RomVersionCompatible(UInt32 requiredVersion)
{
	UInt32 romVersion;

	// See if we're on in minimum required version of the ROM or later.
	FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romVersion);
	if (romVersion < requiredVersion)
	{
		FrmAlert (kRomIncompatibleAlert);  
		return (sysErrRomIncompatible);
	}

	return 0;
}	/* Utils_RomVersionCompatible */
